import yaml
from pymusiclooper.audio import MLAudio


class SampleRateAnalyzer:
    file: str
    output: str

    def __init__(self, yaml_data: dict, output_path: str):

        self.file = yaml_data["File"]
        self.output = output_path


    def run(self) -> bool:

        print("Running sample rate analysis")

        sample_rate: int = 0
        duration: int = 0
        successful: bool
        error: str = ""

        try:
            audio_file = MLAudio(self.file)
            sample_rate = audio_file.rate
            duration = audio_file.total_duration
            successful = True
        except Exception as e:
            print("Error analyzing audio", e)
            successful = False
            error = str(e)

        data = dict(
            Successful=successful,
            Error=error,
            SampleRate=sample_rate,
            Duration=duration
        )

        try:
            with open(self.output, 'w') as outfile:
                yaml.dump(data, outfile, default_flow_style=False)
            return successful
        except Exception as e:
            print(e)
            return False



import importlib.metadata

_DISTRIBUTION_METADATA = importlib.metadata.metadata('py-msu-scripter-app')


class Version:

    @staticmethod
    def name():
        return _DISTRIBUTION_METADATA['Version']


import logging
import sys
import argparse
import yaml
import os.path

from pymusiclooper.audio import MLAudio

from py_msu_scripter_app.py_music_looper_runner import PyMusicLooperRunner
from py_msu_scripter_app.sample_rate_analyzer import SampleRateAnalyzer
from video_creator import VideoCreator
from version import Version

def cli():
    try:
        parser = argparse.ArgumentParser("py_music_scripter_app",
                                         description="Commandline Python application for the MSU Scripter app")
        parser.add_argument("-i", "--input", help="Input YAML file with a list of all PCM files to include", type=str)
        parser.add_argument("-o", "--output", help="The output mp4 file to create", type=str)
        parser.add_argument("-v", "--version", help="Get the version number", action='store_true')
        args = parser.parse_args()

        if args.version:
            print("py_music_scripter_app v" + Version.name())
            exit(1)

        if not args.input:
            print("usage: msu_test_video_creator [-h] [-f FILES] [i INPUT] [-o OUTPUT] [-v]")
            print("msu_test_video_creator: error: you must include either the argument -f/--files or -i/--input")
            exit(1)

        if not args.output:
            print("usage: msu_test_video_creator [-h] [-f FILES] [i INPUT] [-o OUTPUT] [-v]")
            print("msu_test_video_creator: error: the following arguments are required: -o/--output")
            exit(1)

        if not os.path.isfile(args.input):
            print("error: the input YAML file was not found")
            exit(1)


        with open(args.input, "r") as stream:
            try:
                yaml_file = yaml.safe_load(stream)
            except yaml.YAMLError as exc:
                print("error: could not parse input YAML file")
                exit(1)

        mode = yaml_file["Mode"]
        result = False

        if mode == "samples":
            samples = SampleRateAnalyzer(yaml_file, args.output)
            result = samples.run()
        elif mode == "create_video":
            video_creator = VideoCreator(yaml_file, args.output)
            result = video_creator.run()
        elif mode == "py_music_looper":
            py_music_looper = PyMusicLooperRunner(yaml_file, args.output)
            result = py_music_looper.run()


        if result:
            exit(0)
        else:
            exit(1)

    except Exception as e:
        logging.error(e)


if __name__ == "__main__":
    cli()


import threading
import time

import yaml
from proglog import ProgressBarLogger

try:
    import json
    import os
    import re
    import sys
    from pydub import AudioSegment
    from moviepy import AudioFileClip, ColorClip
except ImportError as e:
    print(e)
    print("Please install the following packages:")
    print("json")
    print("os")
    print("pydub")
    print("moviepy")
    exit()


class VideoCreator:
    files: list
    output_video: str
    output: str
    phase: int = 0
    phase_progress: float = 0

    def __init__(self, yaml_data, output):
        self.files = yaml_data["Files"]
        self.output_video = yaml_data["OutputVideo"]
        self.output = output

    def run(self) -> bool:

        if not self.files:
            return self.print_yaml(False, "No tracks marked for inclusion")

        pattern = re.compile("\\.mp4$", re.IGNORECASE)
        pattern2 = re.compile("\\.yml", re.IGNORECASE)

        if not self.output_video:
            return self.print_yaml(False, "Output video file missing")

        if not pattern.findall(self.output_video):
            return self.print_yaml(False, "Invalid output file")

        for track_file in self.files:
            if not os.path.exists(track_file):
                return self.print_yaml(False, f"Track file {track_file} does not exist. Exiting.")

        progress_file = pattern2.sub("_progress.txt", self.output)

        print(progress_file)

        stop_event = threading.Event()
        t = threading.Thread(target=self.writer_thread, args=(progress_file, stop_event))

        try:
            t.start()

            output_mp4 = self.output_video
            output_wav = pattern.sub(".wav", output_mp4)

            print(f"Writing to {output_wav} and {output_mp4}")

            combined_track = AudioSegment.empty()
            for track_file in self.files:
                audio = AudioSegment.from_file(track_file, format="pcm", frame_rate=44100, channels=2, sample_width=2)
                combined_track += audio

            combined_track.export(output_wav, format="wav")

            logger = MyBarLogger(self)

            audio_clip = AudioFileClip(output_wav)
            video_clip = ColorClip(size=(720, 576), color=(0, 0, 0), duration=audio_clip.duration)
            video_clip = video_clip.with_audio(audio_clip)
            video_clip.write_videofile(output_mp4, fps=24, logger=logger)

            return self.print_yaml(True, "")
        except Exception as e:
            print(f"Error creating wav or mp4 file: {str(e)}", file=sys.stderr)
            return self.print_yaml(False, f"Error creating wav or mp4 file {str(e)}")
        finally:
            stop_event.set()
            t.join()


    def print_yaml(self, successful: bool, error: str) -> bool:
        data = dict(
            Successful=successful,
            Error=error,
        )

        try:
            with open(self.output, 'w') as outfile:
                yaml.dump(data, outfile, default_flow_style=False)
            return successful
        except Exception as e:
            print(e)
            return False

    def writer_thread(self, filename, stop_event):
        while not stop_event.is_set():
            with open(filename, "w") as f:
                print(f"writer thread {self.phase} {self.phase_progress}/100")
                f.write(f"{self.phase}|{self.phase_progress}\n")
                f.flush()  # ensure immediate write
                time.sleep(0.5)  # 500 ms


class MyBarLogger(ProgressBarLogger):
    source: VideoCreator

    def __init__(self, source: VideoCreator):
        super().__init__()
        self.source = source

    def bars_callback(self, bar, attr, value, old_value=None):
        percentage = (value / self.bars[bar]['total']) * 100
        if bar == "chunk":
            self.source.phase = 1
            self.source.phase_progress = percentage
            # print(f"Audio: {percentage}/100")
        elif bar == "frame_index":
            self.source.phase = 2
            self.source.phase_progress = percentage
            # print(f"Video: {percentage}/100")



import os

import yaml
from pymusiclooper.core import MusicLooper


class PyMusicLooperRunner:
    file: str
    output: str
    min_duration_multiplier: float = 0.35
    min_loop_duration: float | None = None
    max_loop_duration: float | None = None
    approx_loop_start: float | None = None
    approx_loop_end: float | None = None
    brute_force: bool = False
    disable_pruning: bool = False


    def __init__(self, yaml_data: dict, output_path: str):
        self.file = yaml_data["File"]
        self.output = output_path
        self.min_duration_multiplier = yaml_data["MinDurationMultiplier"]
        self.min_loop_duration = yaml_data["MinLoopDuration"]
        self.max_loop_duration = yaml_data["MaxLoopDuration"]
        self.approx_loop_start = yaml_data["ApproxLoopStart"]
        self.approx_loop_end = yaml_data["ApproxLoopEnd"]


    def run(self):
        if not self.file or not os.path.exists(self.file):
            return self.print_yaml(False, "No input file found")

        try:
            print("Running PyMusicLooper")
            looper = MusicLooper(self.file)
            pairs = looper.find_loop_pairs(
                self.min_duration_multiplier,
                self.min_loop_duration,
                self.max_loop_duration,
                self.approx_loop_start,
                self.approx_loop_end,
                self.brute_force,
                self.disable_pruning)
            pair_objects = []
            for pair in pairs:
                pair_dict = dict(
                    LoopStart=pair.loop_start,
                    LoopEnd=pair.loop_end,
                    Score=float(pair.score),
                    NoteDistance=pair.note_distance,
                    LoudnessDifference=pair.loudness_difference
                )
                pair_objects.append(pair_dict)
            return self.print_yaml(True, "", pair_objects)
        except Exception as e:
            print(f"Error running PyMusicLooper: {str(e)}")
            return self.print_yaml(False, f"Error running PyMusicLooper: {str(e)}")


    def print_yaml(self, successful: bool, error: str, pairs: list | None = None) -> bool:
        if not pairs:
            pairs = []

        data = dict(
            Successful=successful,
            Error=error,
            Pairs=pairs
        )

        try:
            with open(self.output, 'w') as outfile:
                yaml.dump(data, outfile, default_flow_style=False)
            return successful
        except Exception as e:
            print(e)
            return False